﻿using FarPoint.Win.Spread;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Reflection;
using System.Text;
using System.Windows.Forms;

namespace CustomBackground
{
    class ImageSpread : FpSpread
    {
        /// <summary>
        /// Spread have an optimzed performance code, so if there is no background image, it will not invalidate the whole spread bound when scrolling
        /// only new rows/column area appear after scroll will be repaint.
        /// This temp image will help it work smooth in case Layout is not Tile
        /// </summary>
        Image tmpBackgroundImage = new Bitmap(1, 1, PixelFormat.Format16bppArgb1555);

        public override Image BackgroundImage
        {
            get
            {
                if (base.BackgroundImage == null)
                    return tmpBackgroundImage;
                else
                    return base.BackgroundImage;
            }
            set
            {
                base.BackgroundImage = value;
            }
        }

        protected override void Dispose(bool disposing)
        {
            tmpBackgroundImage.Dispose();
            base.Dispose(disposing);
        }

        protected override void OnPaintBackground(PaintEventArgs e)
        {
            base.OnPaintBackground(e); //need to paint for header area and background in case sheetview might contain transparent image/color and need to make it effects.
            ImageSheetView sv = this.ActiveSheet as ImageSheetView;

            if (sv != null)
            {
                Color backColor = sv.BackColor;
                Image image = sv.BackgroundImage;
                ImageLayout layout = sv.BackgroundImageLayout;
                if (backColor == Color.Empty && image == null)
                    return;
                GraphicsState state = e.Graphics.Save();

                Brush br = backColor != Color.Empty && layout == ImageLayout.Tile ? new SolidBrush(backColor) : null;
                MethodInfo drawBackgroundImage = typeof(ControlPaint).GetMethod("DrawBackgroundImage", BindingFlags.NonPublic | BindingFlags.Static, null
                  , new Type[] {typeof(Graphics),typeof(Image),typeof(Color),typeof(ImageLayout),
          typeof(Rectangle),typeof(Rectangle),typeof(Point),typeof(RightToLeft) }, null);

                int rViewportCnt = this.GetRowViewportCount(), cViewportCnt = this.GetColumnViewportCount();
                for (int rowViewport = 0; rowViewport < rViewportCnt; rowViewport++)
                    for (int colViewport = 0; colViewport < cViewportCnt; colViewport++)
                    {
                        int topRow, leftCol;
                        if ((topRow = this.GetViewportTopRow(rowViewport)) < sv.RowCount - 1 && (leftCol = this.GetViewportLeftColumn(colViewport)) < sv.ColumnCount - 1) // Implemented it for total no. of rows and columns rather than a cell range
                        {
                            int n;
                            Rectangle vpRect = this.GetViewportRectangle(rowViewport, colViewport);
                            bool needOffsetPoint = image != null && layout == ImageLayout.Tile;
                            Point scrollOffset = vpRect.Location;
                            if (needOffsetPoint && topRow > 0)
                                for (n = 0; n < topRow; n++)
                                    scrollOffset.Y -= (int)(sv.GetRowHeight(n) * this.ZoomFactor);
                            if (needOffsetPoint && leftCol > 0)
                                for (n = 0; n < leftCol; n++)
                                    scrollOffset.X -= (int)(sv.GetRowHeight(n) * this.ZoomFactor);
                            if (br != null)
                                e.Graphics.FillRectangle(br, vpRect);
                            drawBackgroundImage.Invoke(null, new object[] { e.Graphics, image, backColor, layout, vpRect, vpRect, scrollOffset, this.RightToLeft });
                            e.Graphics.Restore(state);
                        }
                    }
                if (br != null)
                    br.Dispose();
                e.Graphics.Restore(state);
            }
        }
    }
    class ImageSheetView : SheetView
    {
        public ImageSheetView()
            : base()
        {
            this.DefaultStyle.BackColor = Color.Transparent;
            this.BackColor = Color.Empty;
            this.BackgroundImage = null;
            this.BackgroundImageLayout = ImageLayout.None;
        }
        public ImageSheetView(Color backColor, Image background, ImageLayout layout)
            : this()
        {
            this.BackColor = backColor;
            this.BackgroundImage = background;
            this.BackgroundImageLayout = layout;
        }

        /// <summary>
        /// Gets or sets custom back color
        /// </summary>
        public Color BackColor { get; set; }

        /// <summary>
        /// Gets or sets custom background image
        /// </summary>
        public Image BackgroundImage { get; set; }
        /// <summary>
        /// Gets or sets custom background image layout
        /// </summary>
        public ImageLayout BackgroundImageLayout { get; set; }
    }
}
